==> js-language-24.1.1-sources.jar: com/oracle/truffle/js/nodes/access/GetSetRecordNode.java <==
/*
 * Copyright (c) 2023, 2023, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * The Universal Permissive License (UPL), Version 1.0
 *
 * Subject to the condition set forth below, permission is hereby granted to any
 * person obtaining a copy of this software, associated documentation and/or
 * data (collectively the "Software"), free of charge and under any and all
 * copyright rights in the Software, and any and all patent rights owned or
 * freely licensable by each licensor hereunder covering either (i) the
 * unmodified Software as contributed to or provided by such licensor, or (ii)
 * the Larger Works (as defined below), to deal in both
 *
 * (a) the Software, and
 *
 * (b) any piece of software and/or hardware listed in the lrgrwrks.txt file if
 * one is included with the Software each a "Larger Work" to which the Software
 * is contributed by such licensors),
 *
 * without restriction, including without limitation the rights to copy, create
 * derivative works of, display, perform, and distribute the Software and make,
 * use, sell, offer for sale, import, export, have made, and have sold the
 * Software and the Larger Work(s), and to sublicense the foregoing rights on
 * either these or other terms.
 *
 * This license is subject to the following condition:
 *
 * The above copyright notice and either this complete permission notice or at a
 * minimum a reference to the UPL must be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.oracle.truffle.js.nodes.access;

import com.oracle.truffle.api.dsl.Cached;
import com.oracle.truffle.api.dsl.ImportStatic;
import com.oracle.truffle.api.dsl.NeverDefault;
import com.oracle.truffle.api.dsl.Specialization;
import com.oracle.truffle.api.profiles.InlinedBranchProfile;
import com.oracle.truffle.js.nodes.JavaScriptBaseNode;
import com.oracle.truffle.js.nodes.cast.JSNumberToDoubleNode;
import com.oracle.truffle.js.nodes.cast.JSToIntegerOrInfinityNode;
import com.oracle.truffle.js.nodes.cast.JSToNumberNode;
import com.oracle.truffle.js.nodes.unary.IsCallableNode;
import com.oracle.truffle.js.runtime.Errors;
import com.oracle.truffle.js.runtime.JSContext;
import com.oracle.truffle.js.runtime.JSRuntime;
import com.oracle.truffle.js.runtime.Strings;
import com.oracle.truffle.js.runtime.builtins.SetRecord;

/**
 * GetSetRecord(obj).
 */
@ImportStatic({Strings.class})
public abstract class GetSetRecordNode extends JavaScriptBaseNode {
    protected final JSContext context;

    protected GetSetRecordNode(JSContext context) {
        this.context = context;
    }

    @NeverDefault
    public static GetSetRecordNode create(JSContext context) {
        return GetSetRecordNodeGen.create(context);
    }

    public abstract SetRecord execute(Object obj);

    @Specialization
    protected SetRecord get(Object obj,
                    @Cached IsObjectNode isObjectNode,
                    @Cached("create(SIZE, context)") PropertyGetNode getSizeNode,
                    @Cached("create(HAS, context)") PropertyGetNode getHasNode,
                    @Cached("create(KEYS, context)") PropertyGetNode getKeysNode,
                    @Cached JSToNumberNode toNumberNode,
                    @Cached JSToIntegerOrInfinityNode toIntegerOrInfinityNode,
                    @Cached JSNumberToDoubleNode numberToDoubleValueNode,
                    @Cached IsCallableNode isCallableNode,
                    @Cached InlinedBranchProfile errorBranch) {

        if (!isObjectNode.executeBoolean(obj)) {
            errorBranch.enter(this);
            throw Errors.createTypeErrorNotAnObject(obj, this);
        }

        Object rawSize = getSizeNode.getValue(obj);
        Object numSize = toNumberNode.execute(rawSize);
        if (JSRuntime.isNaN(numSize)) {
            errorBranch.enter(this);
            throw Errors.createTypeError("The .size property is NaN", this);
        }
        Object intSize = toIntegerOrInfinityNode.execute(numSize);
        double doubleSize = numberToDoubleValueNode.execute(this, intSize);
        if (doubleSize < 0) {
            errorBranch.enter(this);
            throw Errors.createRangeError("The .size property is negative", this);
        }

        Object has = getHasNode.getValue(obj);
        if (!isCallableNode.executeBoolean(has)) {
            errorBranch.enter(this);
            throw Errors.createTypeErrorCallableExpected();
        }

        Object keys = getKeysNode.getValue(obj);
        if (!isCallableNode.executeBoolean(keys)) {
            errorBranch.enter(this);
            throw Errors.createTypeErrorCallableExpected();
        }

        return new SetRecord(obj, doubleSize, has, keys);
    }

}

==> js-language-25.0.0-sources.jar: com/oracle/truffle/js/nodes/access/GetSetRecordNode.java <==
/*
 * Copyright (c) 2023, 2025, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * The Universal Permissive License (UPL), Version 1.0
 *
 * Subject to the condition set forth below, permission is hereby granted to any
 * person obtaining a copy of this software, associated documentation and/or
 * data (collectively the "Software"), free of charge and under any and all
 * copyright rights in the Software, and any and all patent rights owned or
 * freely licensable by each licensor hereunder covering either (i) the
 * unmodified Software as contributed to or provided by such licensor, or (ii)
 * the Larger Works (as defined below), to deal in both
 *
 * (a) the Software, and
 *
 * (b) any piece of software and/or hardware listed in the lrgrwrks.txt file if
 * one is included with the Software each a "Larger Work" to which the Software
 * is contributed by such licensors),
 *
 * without restriction, including without limitation the rights to copy, create
 * derivative works of, display, perform, and distribute the Software and make,
 * use, sell, offer for sale, import, export, have made, and have sold the
 * Software and the Larger Work(s), and to sublicense the foregoing rights on
 * either these or other terms.
 *
 * This license is subject to the following condition:
 *
 * The above copyright notice and either this complete permission notice or at a
 * minimum a reference to the UPL must be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.oracle.truffle.js.nodes.access;

import com.oracle.truffle.api.dsl.Cached;
import com.oracle.truffle.api.dsl.ImportStatic;
import com.oracle.truffle.api.dsl.NeverDefault;
import com.oracle.truffle.api.dsl.Specialization;
import com.oracle.truffle.api.profiles.InlinedBranchProfile;
import com.oracle.truffle.js.nodes.JavaScriptBaseNode;
import com.oracle.truffle.js.nodes.cast.JSNumberToDoubleNode;
import com.oracle.truffle.js.nodes.cast.JSToIntegerOrInfinityNode;
import com.oracle.truffle.js.nodes.cast.JSToNumberNode;
import com.oracle.truffle.js.nodes.unary.IsCallableNode;
import com.oracle.truffle.js.runtime.Errors;
import com.oracle.truffle.js.runtime.JSContext;
import com.oracle.truffle.js.runtime.JSRuntime;
import com.oracle.truffle.js.runtime.Strings;
import com.oracle.truffle.js.runtime.builtins.SetRecord;

/**
 * GetSetRecord(obj).
 */
@ImportStatic({Strings.class})
public abstract class GetSetRecordNode extends JavaScriptBaseNode {
    protected final JSContext context;

    protected GetSetRecordNode(JSContext context) {
        this.context = context;
    }

    @NeverDefault
    public static GetSetRecordNode create(JSContext context) {
        return GetSetRecordNodeGen.create(context);
    }

    public abstract SetRecord execute(Object obj);

    @Specialization
    protected SetRecord get(Object obj,
                    @Cached IsObjectNode isObjectNode,
                    @Cached("create(SIZE, context)") PropertyGetNode getSizeNode,
                    @Cached("create(HAS, context)") PropertyGetNode getHasNode,
                    @Cached("create(KEYS, context)") PropertyGetNode getKeysNode,
                    @Cached JSToNumberNode toNumberNode,
                    @Cached JSToIntegerOrInfinityNode toIntegerOrInfinityNode,
                    @Cached JSNumberToDoubleNode numberToDoubleValueNode,
                    @Cached IsCallableNode isCallableNode,
                    @Cached InlinedBranchProfile errorBranch) {

        if (!isObjectNode.executeBoolean(obj)) {
            errorBranch.enter(this);
            throw Errors.createTypeErrorNotAnObject(obj, this);
        }

        Object rawSize = getSizeNode.getValue(obj);
        Object numSize = toNumberNode.execute(rawSize);
        if (JSRuntime.isNaN(numSize)) {
            errorBranch.enter(this);
            throw Errors.createTypeError("The .size property is NaN", this);
        }
        Object intSize = toIntegerOrInfinityNode.execute(numSize);
        double doubleSize = numberToDoubleValueNode.execute(this, intSize);
        if (doubleSize < 0) {
            errorBranch.enter(this);
            throw Errors.createRangeErrorFormat("'%s' is an invalid size", this, JSRuntime.safeToString(intSize));
        }

        Object has = getHasNode.getValue(obj);
        if (!isCallableNode.executeBoolean(has)) {
            errorBranch.enter(this);
            throw Errors.createTypeErrorCallableExpected();
        }

        Object keys = getKeysNode.getValue(obj);
        if (!isCallableNode.executeBoolean(keys)) {
            errorBranch.enter(this);
            throw Errors.createTypeErrorCallableExpected();
        }

        return new SetRecord(obj, doubleSize, has, keys);
    }

}
